/*
* Copyright (C) Ghent University - IBBT. All rights reserved.
*
* "This file is part of the IDRA framework (http://idraproject.net)
*
* IDRA is an open-source framework, developed for easy integration and 
* efficient combination of network protocols for netwerked embedded systems.
*   
* IDRA is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 2 of the License, or
* (at your option) any later version.
*
* IDRA is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with IDRA.  If not, see <http://www.gnu.org/licenses/>."
*/

	/**
	* This file describes describes how packet types are constructed 
	* 
	* Each packet has a unique *packet type*, which describes how the packet is constructed.
	* The packet type describes which sequence of packet part descriptors should be used to interpret the packet.
	*
	* For example, a 6lowpan packet type is constructed of the following 3 packet parts: 802.15.4 header + 6 lowpan header + parameter list
	*
	* It is possible to use the neighbor table to set the required packet type for each neighboring node, the system
	* will automatically convert the packet to the correct type. 
	*
	**/


	/**
	* Step 1
	* -------
	* Add the name and ID of new packet types or packet parts
	* Specify how many packet types are used on this node
	**/

	#ifdef PACKET_TYPE_SETTINGS
			enum{
				// 1. General settings
				TOTAL_NUMBER_OF_PACKET_TYPES=2,
				MAX_NUMBER_OF_PARTS_IN_A_PACKET=4,

				// 2. List here all available packet part descriptors (each should be unique)
				IEEE_802_15_4_HEADER = 0, 
				_6LOWPAN_HEADER = 1,	      
				PARAMETER_LIST = 2,

				// 3. List here all availabe packet types (should be unique network wide)  
				_6LOWPAN_PACKET = 0,
				USB_PACKET = 1,

				// 4. Which of these packet types should be used as default (ie: when no specific packet type is entered in the neigbor table)
				DEFAULT_PACKET_TYPE = _6LOWPAN_PACKET,
			};
	#endif

		

	/**
	* Step 2
	* -------
	* Describe from which packet parts each used packet type is constructed
	**/
		
	#ifdef PACKET_TYPE_DEFINITION

		//  now define how each packet type is constructed
		// There should be TOTAL_NUMBER_OF_PACKET_TYPES packets described, each with 
		// MAX_NUMBER_OF_PARTS_IN_A_PACKET  (the unused packet parts have ID -1)
		// Currently, only the last packet part is allowed to have a variable length.
			PacketPartslist[0].packetTypeIdentifier=_6LOWPAN_PACKET;
			PacketPartslist[0].PacketPart[0]=IEEE_802_15_4_HEADER;
			PacketPartslist[0].PacketPart[1]=_6LOWPAN_HEADER;
			PacketPartslist[0].PacketPart[2]=PARAMETER_LIST;
			PacketPartslist[0].PacketPart[3]=-1;

			PacketPartslist[1].packetTypeIdentifier=USB_PACKET;
			PacketPartslist[1].PacketPart[0]=IEEE_802_15_4_HEADER;
			PacketPartslist[1].PacketPart[1]=_6LOWPAN_HEADER;
			PacketPartslist[1].PacketPart[2]=PARAMETER_LIST;
			PacketPartslist[1].PacketPart[3]=-1;
	#endif



	/**
	* Step 3
	* -------
	* Make sure that the packet facade can find the implementation of each used packet part
	**/
	#ifdef PACKET_PART_WIRING	
		// Wiring of PARAMETER_LIST packet part descriptor
		components PacketPart_parListC;
		PacketFacadeP.PacketPart -> PacketPart_parListC;

		// Wiring of IEEE_802_15_4_HEADER packet part descriptor
		components PacketPart_802_15_4_C;
		PacketFacadeP.PacketPart -> PacketPart_802_15_4_C;	

		// Wiring of _6LOWPAN_HEADER packet part descriptor
		components PacketPart_6lowpanC;
		PacketFacadeP.PacketPart ->  PacketPart_6lowpanC;
	#endif

