/*
* Copyright (C) Ghent University - IBBT. All rights reserved.
*
* "This file is part of the IDRA framework (http://idraproject.net)
*
* IDRA is an open-source framework, developed for easy integration and 
* efficient combination of network protocols for netwerked embedded systems.
*   
* IDRA is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 2 of the License, or
* (at your option) any later version.
*
* IDRA is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with IDRA.  If not, see <http://www.gnu.org/licenses/>."
*/

#ifndef PARAMETERS_H
#define PARAMETERS_H

/*************************************************************
*
* 1. Definition of packet attributes (eg: 'header fields')
*	(temporary attributes associated with a packet, that are updated in intermediate nodes through the PacketFacade)
* 2. Definition of information parameters (eg: 'payload')
* 	(parameters that remain unchanged until they reach their destination node)
* 3. Definition of cross-layer parameters
*	(includes parameters between modules, parameters in the neighbor repository and paramaters in the application repository)
*
**************************************************************/


// 1. Definition of the packet attributes (ID between 1 and 100)
enum {
	DESTINATION = 1,  		// size: 16 bit (address) (unsigned). 
						// An individual destination node has an address between 0 and (2^16)-2.
						// To address all neighbors, destination NEIGHBOR_ADDR (or 0) is used
						// To address all nodess, destination BROADCAST_ADDR (or 2^16)-1 is used
	SOURCE = 2,			// size: 16 bit (address) (unsigned)
						// the source address is assigned and updated automatically by the system.
	NEXT_HOP = 3,			// size: 16 bit (address) (unsigned). 
						// Unicast or broadcast (BROADCAST_ADDR) is supported.
	SENDER = 4,			// size: 16 bit (address) (unsigned)
						// the sender address is assigned and updated automatically by the system.
	PACKET_ID = 5,			// size: 8 bit (unsigned)
						// the packet id is assigned and updated automatically by the system.
	TRAFFIC_TYPE_INDICATION = 6,	// size: 8 bit (unsigned). Used when sending information. 
						// This field is associated with each created packet type. Possible values:
						// MAC_INFORMATION,  ROUTING_INFORMATION, MANAGEMENT_INFORMATION, MONITORING_INFORMATION or APPLICATION_INFORMATION.
	TIME_TO_LIVE = 7,		// size: 8 bit (unsigned)


	PRIORITY = 8,			// Indication of the priority of the packet. Size: uint8_t
						// MAC packets: priority 9. Routing packets: priority 8. Monitoring: priority 7.
						// Other applications: priority 1-6.
	RELIABILITY = 9,		// Indication of the reliability of the packet. Size: uint8_t
	CURRENT_DELAY = 10,	// Indication of the total delay of the packet. Size: uint32_t
	MAX_DELAY = 11, 		// Indication of the max delay of the packet. Size: uint32_t

	SOURCE_PREFIX = 20,	// These are not used yet, but enables IDRA to use prefixes in the network (similar to 6lowpan prefixes)
	SENDER_PREFIX = 21,
	DESTINATION_PREFIX = 22,

	ROUTE_HEADER = 30,	// Source routing header. size: 32 bit + 16/address

	AM_ADDRESS = 40,		// Used by the PacketTypeFiltering module to filter packets from different users / deployments (the value of the AM_address is set in Settings.h)
};


// 2. Definition of the information parameters, send to a remote node (ID between 100 and 200)
enum {
	RSSI = 100,			// size: 8 bit (signed), 
					// parameter is automatically generated by the system for each incoming packet
	LQI = 101,			// size: 8 bit (unsigned), 
					// parameter is automatically generated by the system for each incoming packet
	AGGREGATED_PARAMETER = 102,    // used by the system to aggregate a parameter with a packet that is routed to the correct destination
	PIGGYBACKED_PARAMETER = 103,    // used by the system to aggregate a parameter with a packet that is routed to the correct next hop

	MAC_BEACON = 105,		// for synchronizing MAC protocols. Size: 0 bit
	MAC_COMMAND = 106,

	USBTRANSCEIVE= 109,		// for sending messages 

	SINK = 110,			// size: 0 bits. parameter is broadcasted by sink node
	ROUTE_REQUEST = 111,		
	ROUTE_REPLY = 112,		
	ROUTE_ERROR = 113,

	NODE_ROLE = 121,
	NODE_ENERGY=122,
	ADVERTISE = 123,		// advertise yourself as an available node. Size: 0 bits

	ROUTER_SOLLICITATION = 150,	// Sollicitation message from regular node to Router. Size: 64 bit
	ROUTER_ADVERTISEMENT = 151,	// Advertisement broadcast message. Size: variable.
	ROUTER_REGISTRATION = 152,	// Registration from registering node to Edge Router
	ROUTER_CONFIRMATION = 153,	// Confirmation from Edge Router to registering node

	HYDRO_TOPOLOGY_REPORT = 160,	// Topology report from each node in the network to Edge Router
	HYDRO_ROUTE_REQUEST = 161,	// Route Request message sent from Edge Router to SIS 
					//(or other device running the Java program that contains the network topology and shortest path algorithm )
	HYDRO_ROUTE_REPLY = 162,	// Route Reply message from SIS to Edge Router, containing the source route
	HYDRO_TOPOLOGY_PACKAGE = 163,	// Topology overview message that will be exchanged periodically between Edge Routers to ensure topology consistency.
	HYDRO_ROUTE_INSTALL = 164,	// Message to install a route between 2 nodes
	HYDRO_DEFAULT_ROUTED = 165,	// Parameter to check if packet was already delivered through the default route or not
	HYDRO_ROUTE_HEADER_HOP_BY_HOP = 166, //Info parameter to install a hop by hop route, starting from the source of the path
	
	SIP_REQUEST = 170,		// Initialize a sip session between 2 nodes
	SIP_RESPONSE = 171,		// confirmation of a sip session

	ICMP_ECHO = 180,
	ICMP_ECHO_HYDRO = 181,
	
	SYMBIONET_PROFILE = 190,
	NETWORK_ADVERTISEMENT = 191,
	NETWORK_ADVERTISE_REPLY = 192,
	SYMBIO_SINK = 193,
	DISCOVERY_PACKET = 194,
	SERVICE_PROPOSAL = 195,
	ACTIVATE_SERVICES = 196,

	// application information	(ID between 200 and 255)
	TEMPERATURE = 200,		// size: 8 bit (unsigned)
	HUMIDITY = 201,			// size: 8 bit (unsigned)
	LIGHT_INTENSITY = 202,
	BENCHMARK_DATA = 217,
	APPLICATION_DATA = 250,		// variable size

};


// 3. Definition of the cross-layer parameters
//	Neighbor parameters, cross-module parameters or application parameters

enum {
	// I. Neighbor parameters, stored in the neighbor repository for each neighboring node
	LINK_STRENGTH = 1,
	PACKET_TYPE = 2,	// Prefered packet implementation for a neighboring node
					// size: 8 bit (unsigned) (see /Settings/packetTypes.h): PARAMETER_LIST, _6LOWPAN_PACKET, ...	
	RADIO_INTERFACE = 3,		// Prefered radio interface for a neighboring node
					// size: 8 bit (unsigned) (see /Settings/CommunicationInterfaces.h): RADIO_CC2420, USB, ...


	// II. Cross-module parameters, stored in the parameter repository
	PACKET_DELAY = 50,		// uint32_t
	PACKET_SIZE = 51,		// uint16_t
	INTER_PACKET_DELAY = 52, 	// uint32_t
	PACKET_RELIABILITY = 53,	// uint16_t


	// III. QoS parameters, stored in the QoS repository
	BLOCK_SIZE = 101,		// uint16_t
	INTER_BLOCK_DELAY = 102,	// uint32_t
	BLOCK_DELAY = 103,		// uint32_t
	BLOCK_LOSS_RATIO = 104,		// uint8_t

};



#endif 
