/*
* Copyright (C) Ghent University - IBBT. All rights reserved.
*
* "This file is part of the IDRA framework (http://idraproject.net)
*
* IDRA is an open-source framework, developed for easy integration and 
* efficient combination of network protocols for netwerked embedded systems.
*   
* IDRA is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 2 of the License, or
* (at your option) any later version.
*
* IDRA is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with IDRA.  If not, see <http://www.gnu.org/licenses/>."
*/

#ifndef STRUCTURES_H
#define STRUCTURES_H

#include "Settings.h"
#define PACKET_TYPE_SETTINGS
#include "PacketTypes.h"
#undef PACKET_TYPE_SETTINGS

// Used structures and constants in the system
//	1. Multi-mode manager
//	2. Special addresses
//	3. Shared queue
//	4. Packet facade


/*****************************************************
* Multi-mode manager
******************************************************/



// Identifier of module types and the information they can sent
enum {
	FILTERING_PROTOCOL=0,
	MAC_PROTOCOL=1,	
	ROUTING_PROTOCOL=2,
	MONITORING_PROTOCOL=3,
	MANAGEMENT_PROTOCOL=4,
	APPLICATION=8,
};


// Possible COMPARISON values
enum {
	EQUALS,	
	LOWER_THEN,
	GREATER_THEN,
	IS_AVAILABLE,
};

/*****************************************************
* Special address values
******************************************************/

enum {
	BROADCAST_ADDR=65535ul,
	NEIGHBOR_ADDR=65534ul,
};


/*****************************************************
* TrafficTypeIndicator for exchanging information (used by the QoS module)
******************************************************/

// Possible values for the 'TrafficTypeIndicator' metadata
typedef enum {
	MAC_INFORMATION = 0,
	ROUTING_INFORMATION = 1,
	MANAGEMENT_INFORMATION=2,
	MONITORING_INFORMATION=3,
	//RESERVED
	//RESERVED
	//RESERVED
	APPLICATION_INFORMATION=7,
} TrafficTypeIndicator_t;



/*****************************************************
* Possible status values for the stored packets
******************************************************/

// Possible values for the 'status' metadata
typedef enum {
	BEING_STORED = 0,
	READY_FOR_PREPROCESSING = 1,
	PREPROCESSING = 2,		
	READY_FOR_PROCESSING = 3,
	PROCESSING = 4,
	READY_FOR_POSTPROCESSING = 5,
	POSTPROCESSING = 6,
	READY_FOR_SENDING = 7,
	SENDING = 8,
	EMPTY = 9,
} PacketProcessingState_t;


// Possible values for the 'activeProtocol' metadata
typedef enum  {
		NO_PACKET = 0,
		READY_FOR_FILTERING = 1,
		FILTERING_BUSY = 2,
		READY_FOR_MONITORING =3,
		MONITORING_BUSY = 4,
		READY_FOR_ROUTING = 5,
		ROUTING_BUSY = 6,
		READY_FOR_MAC = 7,
		MAC_BUSY = 8,
		READY_FOR_FINISHING = 9,
		FINISHED = 10,
} ActiveProtocol_t;


/**************************************************************
 * Structures for interacting with the shared queue
 *************************************************************/

//Intern packets from the shared queue: 
//	stored as the packet + metadata
typedef struct {
	uint8_t		interfaceType;
	uint32_t 		timeOfArrival;	// arrival time of the packet
	int8_t		rssi;		// rssi of the packet
	uint8_t		lqi;		// link quality of the packet
	uint16_t		frequency;
	bool			crcOk;
	bool			createdLocally; // TRUE if the packet was created locally, FALSE if it was received from a remote node
} rxMetaData_t;

typedef struct {
	uint8_t			interfaceType;
	//localtime_t 		timeOfTransmit;
	//int8_t		txPower;
 	//uint8_t		CCA;
	//uint16_t		frequency; //In MHz
	//bool			clearAfterTransmit;
} txMetaData_t;


typedef struct {
	//Status data + info about the current active protocol
  	PacketProcessingState_t status;
	ActiveProtocol_t	activeProtocol;   

	// associated metadata
	 rxMetaData_t		rxMetadata;
	 txMetaData_t		txMetadata;

	// Information about the content of the packet
	int8_t			packetType;
	int8_t			packetPartType[MAX_NUMBER_OF_PARTS_IN_A_PACKET];
	int8_t			packetPartSize[MAX_NUMBER_OF_PARTS_IN_A_PACKET];

	// The actual packet
	uint8_t			storedPacket[MAXPACKETSIZE+MAXMETADATASIZE];
} internalPacket_t;


/**************************************************************
 * Structures for converting packets to parameter arrays
 *************************************************************/

// representation of packet parameters
typedef struct {
	uint16_t 	parameterName;				// parameter ID
	int8_t		parameterSize;				// parameter size in bytes
	uint8_t* 	parameterLocation;
}  packetParameter;	


typedef  packetParameter	parameterArray[MAXPACKETPARAMETERS]; 	

/**************************************************************
 * Structures for defining aggregation and piggybacked parameters
 *************************************************************/


// representation of an aggregation parameter
typedef struct {
	uint16_t 	parameterSource;		
	uint16_t 	parameterName;	
	uint8_t 	parameterSize;			
	uint8_t 	parameterData[MAXPARAMETERSIZE];;
}  aggregatedParameter;	




#endif
